"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.registerHook = exports.getElementUpdateFn = exports.getCurrentElement = exports.renderWithHooks = void 0;
let hooksState;
const OnRender = (element, elementUpdateFn) => {
    hooksState = {
        element,
        renderIsInProgress: true,
        hooksExecutedThisRender: 0,
        elementUpdateFn,
    };
};
const OnEndRender = () => {
    hooksState = {
        element: null,
        renderIsInProgress: false,
        hooksExecutedThisRender: 0,
        elementUpdateFn: null,
    };
};
const renderWithHooks = (element, templateFn, elementUpdateFn) => {
    OnRender(element, elementUpdateFn);
    const template = templateFn(element);
    OnEndRender();
    return template;
};
exports.renderWithHooks = renderWithHooks;
const getCurrentElement = () => hooksState.element;
exports.getCurrentElement = getCurrentElement;
const getElementUpdateFn = () => hooksState.elementUpdateFn;
exports.getElementUpdateFn = getElementUpdateFn;
const registerHook = (name) => {
    if (!hooksState.renderIsInProgress) {
        throw new Error(`Hooks must be called from within an element's render() method. ${name} was not.`);
    }
    const hookId = hooksState.hooksExecutedThisRender++;
    return [hooksState.element, hookId];
};
exports.registerHook = registerHook;
//# sourceMappingURL=hooks-core.js.map