"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useState = void 0;
const hooks_core_1 = require("./hooks-core");
const stateMap = new WeakMap();
const updateState = (element, hookID, elementUpdateFn, newValue) => {
    const currentValue = stateMap.get(element).get(hookID);
    if (currentValue === newValue) {
        return;
    }
    stateMap.get(element).set(hookID, newValue);
    if (elementUpdateFn) {
        elementUpdateFn();
    }
};
const useState = (value) => {
    const [element, hookID] = (0, hooks_core_1.registerHook)();
    const elementUpdateFn = (0, hooks_core_1.getElementUpdateFn)();
    if (!stateMap.has(element)) {
        stateMap.set(element, new Map());
    }
    if (!stateMap.get(element).has(hookID)) {
        updateState(element, hookID, null, value);
    }
    return [
        stateMap.get(element).get(hookID),
        updateState.bind(null, element, hookID, elementUpdateFn),
    ];
};
exports.useState = useState;
//# sourceMappingURL=use-state.js.map